@@ -38,7 +38,13 @@ class Document(object):
     """
     __slots__ = ('text', 'cursor_position', 'selection')
 
-    def __init__(self, text='', cursor_position=0, selection=None):
+    def __init__(self, text='', cursor_position=None, selection=None):
+        # By default, if no cursor position was given, make sure to put the
+        # cursor position is at the end of the document. This is what makes
+        # sense in most places.
+        if cursor_position is None:
+            cursor_position = len(text)
+
         self.text = text
         self.cursor_position = cursor_position
         self.selection = selection
@@ -219,7 +225,7 @@ class Document(object):
         except StopIteration:
             pass
 
-    def get_word_before_cursor(self):
+    def get_word_before_cursor(self, WORD=False):
         """
         Give the word before the cursor.
         If we have whitespace before the cursor this returns an empty string.
@@ -227,7 +233,7 @@ class Document(object):
         if self.text_before_cursor[-1:].isspace():
             return ''
         else:
-            return self.text_before_cursor[self.find_start_of_previous_word():]
+            return self.text_before_cursor[self.find_start_of_previous_word(WORD=WORD):]
 
     def find_start_of_previous_word(self, count=1, WORD=False):
         """
